package net.northking.cctp.element.util;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class taken from Spring Framework.
 * @author
 * @since 1.0
 */
public class AntPathMatcher {

	private static final Pattern VARIABLE_PATTERN = Pattern.compile("\\{[^/]+?\\}");

	/** Default path separator: "/" */
	public static final String DEFAULT_PATH_SEPARATOR = "/";

	private String pathSeparator = DEFAULT_PATH_SEPARATOR;


	/** Set the path separator to use for pattern parsing. Default is "/", as in Ant. */
	public void setPathSeparator(String pathSeparator) {
		this.pathSeparator = (pathSeparator != null ? pathSeparator : DEFAULT_PATH_SEPARATOR);
	}


	public boolean isPattern(String path) {
		return (path.indexOf('*') != -1 || path.indexOf('?') != -1);
	}

	public boolean match(String pattern, String path) {
		return doMatch(pattern, path, true, null);
	}

	public boolean matchStart(String pattern, String path) {
		return doMatch(pattern, path, false, null);
	}

    /**
     * Borrowed from PathMatchingResourcePatternResolver
     */
    public String getRoot(String pattern) {
        int prefixEnd = pattern.indexOf(":") + 1;
        int rootDirEnd = pattern.length();
        while (rootDirEnd > prefixEnd && isPattern(pattern.substring(prefixEnd, rootDirEnd))) {
            rootDirEnd = pattern.lastIndexOf('/', rootDirEnd - 2) + 1;
        }
        if (rootDirEnd == 0) {
            rootDirEnd = prefixEnd;
        }
        return pattern.substring(0, rootDirEnd);
    }

	/**
	 * Actually match the given <code>path</code> against the given <code>pattern</code>.
	 * @param pattern the pattern to match against
	 * @param path the path String to test
	 * @param fullMatch whether a full pattern match is required (else a pattern match
	 * as far as the given base path goes is sufficient)
	 * @return <code>true</code> if the supplied <code>path</code> matched, <code>false</code> if it didn't
	 */
	protected boolean doMatch(String pattern, String path, boolean fullMatch,
			Map<String, String> uriTemplateVariables) {

		if (path.startsWith(this.pathSeparator) != pattern.startsWith(this.pathSeparator)) {
			return false;
		}

		String[] pattDirs = StringUtils.tokenizeToStringArray(pattern, this.pathSeparator);
		String[] pathDirs = StringUtils.tokenizeToStringArray(path, this.pathSeparator);

		int pattIdxStart = 0;
		int pattIdxEnd = pattDirs.length - 1;
		int pathIdxStart = 0;
		int pathIdxEnd = pathDirs.length - 1;

		// Match all elements up to the first **
		while (pattIdxStart <= pattIdxEnd && pathIdxStart <= pathIdxEnd) {
			String patDir = pattDirs[pattIdxStart];
			if ("**".equals(patDir)) {
				break;
			}
			if (!matchStrings(patDir, pathDirs[pathIdxStart], uriTemplateVariables)) {
				return false;
			}
			pattIdxStart++;
			pathIdxStart++;
		}

		if (pathIdxStart > pathIdxEnd) {
			// Path is exhausted, only match if rest of pattern is * or **'s
			if (pattIdxStart > pattIdxEnd) {
				return (pattern.endsWith(this.pathSeparator) ? path.endsWith(this.pathSeparator) :
						!path.endsWith(this.pathSeparator));
			}
			if (!fullMatch) {
				return true;
			}
			if (pattIdxStart == pattIdxEnd && pattDirs[pattIdxStart].equals("*") && path.endsWith(this.pathSeparator)) {
				return true;
			}
			for (int i = pattIdxStart; i <= pattIdxEnd; i++) {
				if (!pattDirs[i].equals("**")) {
					return false;
				}
			}
			return true;
		}
		else if (pattIdxStart > pattIdxEnd) {
			// String not exhausted, but pattern is. Failure.
			return false;
		}
		else if (!fullMatch && "**".equals(pattDirs[pattIdxStart])) {
			// Path start definitely matches due to "**" part in pattern.
			return true;
		}

		// up to last '**'
		while (pattIdxStart <= pattIdxEnd && pathIdxStart <= pathIdxEnd) {
			String patDir = pattDirs[pattIdxEnd];
			if (patDir.equals("**")) {
				break;
			}
			if (!matchStrings(patDir, pathDirs[pathIdxEnd], uriTemplateVariables)) {
				return false;
			}
			pattIdxEnd--;
			pathIdxEnd--;
		}
		if (pathIdxStart > pathIdxEnd) {
			// String is exhausted
			for (int i = pattIdxStart; i <= pattIdxEnd; i++) {
				if (!pattDirs[i].equals("**")) {
					return false;
				}
			}
			return true;
		}

		while (pattIdxStart != pattIdxEnd && pathIdxStart <= pathIdxEnd) {
			int patIdxTmp = -1;
			for (int i = pattIdxStart + 1; i <= pattIdxEnd; i++) {
				if (pattDirs[i].equals("**")) {
					patIdxTmp = i;
					break;
				}
			}
			if (patIdxTmp == pattIdxStart + 1) {
				// '**/**' situation, so skip one
				pattIdxStart++;
				continue;
			}
			// Find the pattern between padIdxStart & padIdxTmp in str between
			// strIdxStart & strIdxEnd
			int patLength = (patIdxTmp - pattIdxStart - 1);
			int strLength = (pathIdxEnd - pathIdxStart + 1);
			int foundIdx = -1;

			strLoop:
			for (int i = 0; i <= strLength - patLength; i++) {
				for (int j = 0; j < patLength; j++) {
					String subPat = pattDirs[pattIdxStart + j + 1];
					String subStr = pathDirs[pathIdxStart + i + j];
					if (!matchStrings(subPat, subStr, uriTemplateVariables)) {
						continue strLoop;
					}
				}
				foundIdx = pathIdxStart + i;
				break;
			}

			if (foundIdx == -1) {
				return false;
			}

			pattIdxStart = patIdxTmp;
			pathIdxStart = foundIdx + patLength;
		}

		for (int i = pattIdxStart; i <= pattIdxEnd; i++) {
			if (!pattDirs[i].equals("**")) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Tests whether or not a string matches against a pattern. The pattern may contain two special characters:<br> '*'
	 * means zero or more characters<br> '?' means one and only one character
	 * @param pattern pattern to match against. Must not be <code>null</code>.
	 * @param str string which must be matched against the pattern. Must not be <code>null</code>.
	 * @return <code>true</code> if the string matches against the pattern, or <code>false</code> otherwise.
	 */
	private boolean matchStrings(String pattern, String str, Map<String, String> uriTemplateVariables) {
		AntPathStringMatcher matcher = new AntPathStringMatcher(pattern, str, uriTemplateVariables);
		return matcher.matchStrings();
	}

	/**
	 * Given a pattern and a full path, determine the pattern-mapped part. <p>For example: <ul>
	 * <li>'<code>/docs/cvs/commit.html</code>' and '<code>/docs/cvs/commit.html</code> -> ''</li>
	 * <li>'<code>/docs/*</code>' and '<code>/docs/cvs/commit</code> -> '<code>cvs/commit</code>'</li>
	 * <li>'<code>/docs/cvs/*.html</code>' and '<code>/docs/cvs/commit.html</code> -> '<code>commit.html</code>'</li>
	 * <li>'<code>/docs/**</code>' and '<code>/docs/cvs/commit</code> -> '<code>cvs/commit</code>'</li>
	 * <li>'<code>/docs/**\/*.html</code>' and '<code>/docs/cvs/commit.html</code> -> '<code>cvs/commit.html</code>'</li>
	 * <li>'<code>/*.html</code>' and '<code>/docs/cvs/commit.html</code> -> '<code>docs/cvs/commit.html</code>'</li>
	 * <li>'<code>*.html</code>' and '<code>/docs/cvs/commit.html</code> -> '<code>/docs/cvs/commit.html</code>'</li>
	 * <li>'<code>*</code>' and '<code>/docs/cvs/commit.html</code> -> '<code>/docs/cvs/commit.html</code>'</li> </ul>
	 * <p>Assumes that {@link #match} returns <code>true</code> for '<code>pattern</code>' and '<code>path</code>', but
	 * does <strong>not</strong> enforce this.
	 */
	public String extractPathWithinPattern(String pattern, String path) {
		String[] patternParts = StringUtils.tokenizeToStringArray(pattern, this.pathSeparator);
		String[] pathParts = StringUtils.tokenizeToStringArray(path, this.pathSeparator);

		StringBuilder builder = new StringBuilder();

		// Add any path parts that have a wildcarded pattern part.
		int puts = 0;
		if (patternParts != null && pathParts != null) {
			for (int i = 0; i < patternParts.length; i++) {
				String patternPart = patternParts[i];
				if ((patternPart.indexOf('*') > -1 || patternPart.indexOf('?') > -1) && pathParts.length >= i + 1) {
					if (puts > 0 || (i == 0 && !pattern.startsWith(this.pathSeparator))) {
						builder.append(this.pathSeparator);
					}
					builder.append(pathParts[i]);
					puts++;
				}
			}
			// Append any trailing path parts.
			for (int i = patternParts.length; i < pathParts.length; i++) {
				if (puts > 0 || i > 0) {
					builder.append(this.pathSeparator);
				}
				builder.append(pathParts[i]);
			}
		}
		return builder.toString();
	}

	public Map<String, String> extractUriTemplateVariables(String pattern, String path) {
		Map<String, String> variables = new LinkedHashMap<String, String>();
		doMatch(pattern, path, true, variables);
		return variables;
	}

	/**
	 * Combines two patterns into a new pattern that is returned.
	 * <p>This implementation simply concatenates the two patterns, unless the first pattern
	 * contains a file extension match (such as {@code *.html}. In that case, the second pattern
	 * should be included in the first, or an {@code IllegalArgumentException} is thrown.
	 * <p>For example: <table>
	 * <tr><th>Pattern 1</th><th>Pattern 2</th><th>Result</th></tr> <tr><td>/hotels</td><td>{@code
	 * null}</td><td>/hotels</td></tr> <tr><td>{@code null}</td><td>/hotels</td><td>/hotels</td></tr>
	 * <tr><td>/hotels</td><td>/bookings</td><td>/hotels/bookings</td></tr> <tr><td>/hotels</td><td>bookings</td><td>/hotels/bookings</td></tr>
	 * <tr><td>/hotels/*</td><td>/bookings</td><td>/hotels/bookings</td></tr> <tr><td>/hotels/&#42;&#42;</td><td>/bookings</td><td>/hotels/&#42;&#42;/bookings</td></tr>
	 * <tr><td>/hotels</td><td>{hotel}</td><td>/hotels/{hotel}</td></tr> <tr><td>/hotels/*</td><td>{hotel}</td><td>/hotels/{hotel}</td></tr>
	 * <tr><td>/hotels/&#42;&#42;</td><td>{hotel}</td><td>/hotels/&#42;&#42;/{hotel}</td></tr>
	 * <tr><td>/*.html</td><td>/hotels.html</td><td>/hotels.html</td></tr> <tr><td>/*.html</td><td>/hotels</td><td>/hotels.html</td></tr>
	 * <tr><td>/*.html</td><td>/*.txt</td><td>IllegalArgumentException</td></tr> </table>
	 * @param pattern1 the first pattern
	 * @param pattern2 the second pattern
	 * @return the combination of the two patterns
	 * @throws IllegalArgumentException when the two patterns cannot be combined
	 */
	public String combine(String pattern1, String pattern2) {
		if (!StringUtils.hasText(pattern1) && !StringUtils.hasText(pattern2)) {
			return "";
		}
		else if (!StringUtils.hasText(pattern1)) {
			return pattern2;
		}
		else if (!StringUtils.hasText(pattern2)) {
			return pattern1;
		}
		else if (!pattern1.equals(pattern2) && !pattern1.contains("{") && match(pattern1, pattern2)) {
			// /* + /hotel -> /hotel ; "/*.*" + "/*.html" -> /*.html
			// However /user + /user -> /usr/user ; /{foo} + /bar -> /{foo}/bar
			return pattern2;
		}
		else if (pattern1.endsWith("/*")) {
			if (pattern2.startsWith("/")) {
				// /hotels/* + /booking -> /hotels/booking
				return pattern1.substring(0, pattern1.length() - 1) + pattern2.substring(1);
			}
			else {
				// /hotels/* + booking -> /hotels/booking
				return pattern1.substring(0, pattern1.length() - 1) + pattern2;
			}
		}
		else if (pattern1.endsWith("/**")) {
			if (pattern2.startsWith("/")) {
				// /hotels/** + /booking -> /hotels/**/booking
				return pattern1 + pattern2;
			}
			else {
				// /hotels/** + booking -> /hotels/**/booking
				return pattern1 + "/" + pattern2;
			}
		}
		else {
			int dotPos1 = pattern1.indexOf('.');
			if (dotPos1 == -1) {
				// simply concatenate the two patterns
				if (pattern1.endsWith("/") || pattern2.startsWith("/")) {
					return pattern1 + pattern2;
				}
				else {
					return pattern1 + "/" + pattern2;
				}
			}
			String fileName1 = pattern1.substring(0, dotPos1);
			String extension1 = pattern1.substring(dotPos1);
			String fileName2;
			String extension2;
			int dotPos2 = pattern2.indexOf('.');
			if (dotPos2 != -1) {
				fileName2 = pattern2.substring(0, dotPos2);
				extension2 = pattern2.substring(dotPos2);
			}
			else {
				fileName2 = pattern2;
				extension2 = "";
			}
			String fileName = fileName1.endsWith("*") ? fileName2 : fileName1;
			String extension = extension1.startsWith("*") ? extension2 : extension1;

			return fileName + extension;
		}
	}

	/**
	 * Given a full path, returns a {@link Comparator} suitable for sorting patterns in order of explicitness.
	 * <p>The returned <code>Comparator</code> will {@linkplain java.util.Collections#sort(java.util.List,
	 * java.util.Comparator) sort} a list so that more specific patterns (without uri templates or wild cards) come before
	 * generic patterns. So given a list with the following patterns: <ol> <li><code>/hotels/new</code></li>
	 * <li><code>/hotels/{hotel}</code></li> <li><code>/hotels/*</code></li> </ol> the returned comparator will sort this
	 * list so that the order will be as indicated.
	 * <p>The full path given as parameter is used to test for exact matches. So when the given path is {@code /hotels/2},
	 * the pattern {@code /hotels/2} will be sorted before {@code /hotels/1}.
	 * @param path the full path to use for comparison
	 * @return a comparator capable of sorting patterns in order of explicitness
	 */
	public Comparator<String> getPatternComparator(String path) {
		return new AntPatternComparator(path);
	}


	private static class AntPatternComparator implements Comparator<String> {

		private final String path;

		private AntPatternComparator(String path) {
			this.path = path;
		}

		public int compare(String pattern1, String pattern2) {
			if (pattern1 == null && pattern2 == null) {
				return 0;
			}
			else if (pattern1 == null) {
				return 1;
			}
			else if (pattern2 == null) {
				return -1;
			}
			boolean pattern1EqualsPath = pattern1.equals(path);
			boolean pattern2EqualsPath = pattern2.equals(path);
			if (pattern1EqualsPath && pattern2EqualsPath) {
				return 0;
			}
			else if (pattern1EqualsPath) {
				return -1;
			}
			else if (pattern2EqualsPath) {
				return 1;
			}
			int wildCardCount1 = getWildCardCount(pattern1);
			int wildCardCount2 = getWildCardCount(pattern2);

			int bracketCount1 = StringUtils.countOccurrencesOf(pattern1, "{");
			int bracketCount2 = StringUtils.countOccurrencesOf(pattern2, "{");

			int totalCount1 = wildCardCount1 + bracketCount1;
			int totalCount2 = wildCardCount2 + bracketCount2;

			if (totalCount1 != totalCount2) {
				return totalCount1 - totalCount2;
			}

			int pattern1Length = getPatternLength(pattern1);
			int pattern2Length = getPatternLength(pattern2);

			if (pattern1Length != pattern2Length) {
				return pattern2Length - pattern1Length;
			}

			if (wildCardCount1 < wildCardCount2) {
				return -1;
			}
			else if (wildCardCount2 < wildCardCount1) {
				return 1;
			}

			if (bracketCount1 < bracketCount2) {
				return -1;
			}
			else if (bracketCount2 < bracketCount1) {
				return 1;
			}

			return 0;
		}

		private int getWildCardCount(String pattern) {
			if (pattern.endsWith(".*")) {
				pattern = pattern.substring(0, pattern.length() - 2);
			}
			return StringUtils.countOccurrencesOf(pattern, "*");
		}

		/**
		 * Returns the length of the given pattern, where template variables are considered to be 1 long.
		 */
		private int getPatternLength(String pattern) {
			Matcher m = VARIABLE_PATTERN.matcher(pattern);
			return m.replaceAll("#").length();
		}
	}




	/**
	 * This class taken from Spring Framework.
	 *
	 * @author
	 * @since 1.0
	 */
	public static class StringUtils {

		private static final String FOLDER_SEPARATOR = "/";

		private static final char EXTENSION_SEPARATOR = '.';

		// ---------------------------------------------------------------------
		// General convenience methods for working with Strings
		// ---------------------------------------------------------------------

		/**
		 * Check that the given CharSequence is neither <code>null</code> nor of length
		 * 0. Note: Will return <code>true</code> for a CharSequence that purely
		 * consists of whitespace.
		 * <p>
		 *
		 * <pre>
		 * StringUtils.hasLength(null) = false
		 * StringUtils.hasLength("") = false
		 * StringUtils.hasLength(" ") = true
		 * StringUtils.hasLength("Hello") = true
		 * </pre>
		 *
		 * @param str the CharSequence to check (may be <code>null</code>)
		 * @return <code>true</code> if the CharSequence is not null and has length
		 * @see #hasText(String)
		 */
		public static boolean hasLength(CharSequence str) {
			return (str != null && str.length() > 0);
		}

		/**
		 * Check that the given String is neither <code>null</code> nor of length 0.
		 * Note: Will return <code>true</code> for a String that purely consists of
		 * whitespace.
		 *
		 * @param str the String to check (may be <code>null</code>)
		 * @return <code>true</code> if the String is not null and has length
		 * @see #hasLength(CharSequence)
		 */
		public static boolean hasLength(String str) {
			return hasLength((CharSequence) str);
		}

		/**
		 * Check whether the given CharSequence has actual text. More specifically,
		 * returns <code>true</code> if the string not <code>null</code>, its length is
		 * greater than 0, and it contains at least one non-whitespace character.
		 * <p>
		 *
		 * <pre>
		 * StringUtils.hasText(null) = false
		 * StringUtils.hasText("") = false
		 * StringUtils.hasText(" ") = false
		 * StringUtils.hasText("12345") = true
		 * StringUtils.hasText(" 12345 ") = true
		 * </pre>
		 *
		 * @param str the CharSequence to check (may be <code>null</code>)
		 * @return <code>true</code> if the CharSequence is not <code>null</code>, its
		 *         length is greater than 0, and it does not contain whitespace only
		 * @see java.lang.Character#isWhitespace
		 */
		public static boolean hasText(CharSequence str) {
			if (!hasLength(str)) {
				return false;
			}
			int strLen = str.length();
			for (int i = 0; i < strLen; i++) {
				if (!Character.isWhitespace(str.charAt(i))) {
					return true;
				}
			}
			return false;
		}

		/**
		 * Check whether the given String has actual text. More specifically, returns
		 * <code>true</code> if the string not <code>null</code>, its length is greater
		 * than 0, and it contains at least one non-whitespace character.
		 *
		 * @param str the String to check (may be <code>null</code>)
		 * @return <code>true</code> if the String is not <code>null</code>, its length
		 *         is greater than 0, and it does not contain whitespace only
		 * @see #hasText(CharSequence)
		 */
		public static boolean hasText(String str) {
			return hasText((CharSequence) str);
		}

		/**
		 * Check whether the given CharSequence contains any whitespace characters.
		 *
		 * @param str the CharSequence to check (may be <code>null</code>)
		 * @return <code>true</code> if the CharSequence is not empty and contains at
		 *         least 1 whitespace character
		 * @see java.lang.Character#isWhitespace
		 */
		public static boolean containsWhitespace(CharSequence str) {
			if (!hasLength(str)) {
				return false;
			}
			int strLen = str.length();
			for (int i = 0; i < strLen; i++) {
				if (Character.isWhitespace(str.charAt(i))) {
					return true;
				}
			}
			return false;
		}

		/**
		 * Check whether the given String contains any whitespace characters.
		 *
		 * @param str the String to check (may be <code>null</code>)
		 * @return <code>true</code> if the String is not empty and contains at least 1
		 *         whitespace character
		 * @see #containsWhitespace(CharSequence)
		 */
		public static boolean containsWhitespace(String str) {
			return containsWhitespace((CharSequence) str);
		}

		/**
		 * Trim leading and trailing whitespace from the given String.
		 *
		 * @param str the String to check
		 * @return the trimmed String
		 * @see java.lang.Character#isWhitespace
		 */
		public static String trimWhitespace(String str) {
			if (!hasLength(str)) {
				return str;
			}
			StringBuilder sb = new StringBuilder(str);
			while (sb.length() > 0 && Character.isWhitespace(sb.charAt(0))) {
				sb.deleteCharAt(0);
			}
			while (sb.length() > 0 && Character.isWhitespace(sb.charAt(sb.length() - 1))) {
				sb.deleteCharAt(sb.length() - 1);
			}
			return sb.toString();
		}

		/**
		 * Trim <i>all</i> whitespace from the given String: leading, trailing, and
		 * inbetween characters.
		 *
		 * @param str the String to check
		 * @return the trimmed String
		 * @see java.lang.Character#isWhitespace
		 */
		public static String trimAllWhitespace(String str) {
			if (!hasLength(str)) {
				return str;
			}
			StringBuilder sb = new StringBuilder(str);
			int index = 0;
			while (sb.length() > index) {
				if (Character.isWhitespace(sb.charAt(index))) {
					sb.deleteCharAt(index);
				} else {
					index++;
				}
			}
			return sb.toString();
		}

		/**
		 * Trim leading whitespace from the given String.
		 *
		 * @param str the String to check
		 * @return the trimmed String
		 * @see java.lang.Character#isWhitespace
		 */
		public static String trimLeadingWhitespace(String str) {
			if (!hasLength(str)) {
				return str;
			}
			StringBuilder sb = new StringBuilder(str);
			while (sb.length() > 0 && Character.isWhitespace(sb.charAt(0))) {
				sb.deleteCharAt(0);
			}
			return sb.toString();
		}

		/**
		 * Trim trailing whitespace from the given String.
		 *
		 * @param str the String to check
		 * @return the trimmed String
		 * @see java.lang.Character#isWhitespace
		 */
		public static String trimTrailingWhitespace(String str) {
			if (!hasLength(str)) {
				return str;
			}
			StringBuilder sb = new StringBuilder(str);
			while (sb.length() > 0 && Character.isWhitespace(sb.charAt(sb.length() - 1))) {
				sb.deleteCharAt(sb.length() - 1);
			}
			return sb.toString();
		}

		/**
		 * Trim all occurences of the supplied leading character from the given String.
		 *
		 * @param str              the String to check
		 * @param leadingCharacter the leading character to be trimmed
		 * @return the trimmed String
		 */
		public static String trimLeadingCharacter(String str, char leadingCharacter) {
			if (!hasLength(str)) {
				return str;
			}
			StringBuilder sb = new StringBuilder(str);
			while (sb.length() > 0 && sb.charAt(0) == leadingCharacter) {
				sb.deleteCharAt(0);
			}
			return sb.toString();
		}

		/**
		 * Trim all occurences of the supplied trailing character from the given String.
		 *
		 * @param str               the String to check
		 * @param trailingCharacter the trailing character to be trimmed
		 * @return the trimmed String
		 */
		public static String trimTrailingCharacter(String str, char trailingCharacter) {
			if (!hasLength(str)) {
				return str;
			}
			StringBuilder sb = new StringBuilder(str);
			while (sb.length() > 0 && sb.charAt(sb.length() - 1) == trailingCharacter) {
				sb.deleteCharAt(sb.length() - 1);
			}
			return sb.toString();
		}

		/**
		 * Test if the given String starts with the specified prefix, ignoring
		 * upper/lower case.
		 *
		 * @param str    the String to check
		 * @param prefix the prefix to look for
		 * @see java.lang.String#startsWith
		 */
		public static boolean startsWithIgnoreCase(String str, String prefix) {
			if (str == null || prefix == null) {
				return false;
			}
			if (str.startsWith(prefix)) {
				return true;
			}
			if (str.length() < prefix.length()) {
				return false;
			}
			String lcStr = str.substring(0, prefix.length()).toLowerCase();
			String lcPrefix = prefix.toLowerCase();
			return lcStr.equals(lcPrefix);
		}

		/**
		 * Test if the given String ends with the specified suffix, ignoring upper/lower
		 * case.
		 *
		 * @param str    the String to check
		 * @param suffix the suffix to look for
		 * @see java.lang.String#endsWith
		 */
		public static boolean endsWithIgnoreCase(String str, String suffix) {
			if (str == null || suffix == null) {
				return false;
			}
			if (str.endsWith(suffix)) {
				return true;
			}
			if (str.length() < suffix.length()) {
				return false;
			}

			String lcStr = str.substring(str.length() - suffix.length()).toLowerCase();
			String lcSuffix = suffix.toLowerCase();
			return lcStr.equals(lcSuffix);
		}

		/**
		 * Test whether the given string matches the given substring at the given index.
		 *
		 * @param str       the original string (or StringBuilder)
		 * @param index     the index in the original string to start matching against
		 * @param substring the substring to match at the given index
		 */
		public static boolean substringMatch(CharSequence str, int index, CharSequence substring) {
			for (int j = 0; j < substring.length(); j++) {
				int i = index + j;
				if (i >= str.length() || str.charAt(i) != substring.charAt(j)) {
					return false;
				}
			}
			return true;
		}

		/**
		 * Count the occurrences of the substring in string s.
		 *
		 * @param str string to search in. Return 0 if this is null.
		 * @param sub string to search for. Return 0 if this is null.
		 */
		public static int countOccurrencesOf(String str, String sub) {
			if (str == null || sub == null || str.length() == 0 || sub.length() == 0) {
				return 0;
			}
			int count = 0;
			int pos = 0;
			int idx;
			while ((idx = str.indexOf(sub, pos)) != -1) {
				++count;
				pos = idx + sub.length();
			}
			return count;
		}

		/**
		 * Replace all occurences of a substring within a string with another string.
		 *
		 * @param inString   String to examine
		 * @param oldPattern String to replace
		 * @param newPattern String to insert
		 * @return a String with the replacements
		 */
		public static String replace(String inString, String oldPattern, String newPattern) {
			if (!hasLength(inString) || !hasLength(oldPattern) || newPattern == null) {
				return inString;
			}
			StringBuilder sb = new StringBuilder();
			int pos = 0; // our position in the old string
			int index = inString.indexOf(oldPattern);
			// the index of an occurrence we've found, or -1
			int patLen = oldPattern.length();
			while (index >= 0) {
				sb.append(inString.substring(pos, index));
				sb.append(newPattern);
				pos = index + patLen;
				index = inString.indexOf(oldPattern, pos);
			}
			sb.append(inString.substring(pos));
			// remember to append any characters to the right of a match
			return sb.toString();
		}

		/**
		 * Delete all occurrences of the given substring.
		 *
		 * @param inString the original String
		 * @param pattern  the pattern to delete all occurrences of
		 * @return the resulting String
		 */
		public static String delete(String inString, String pattern) {
			return replace(inString, pattern, "");
		}

		/**
		 * Delete any character in a given String.
		 *
		 * @param inString      the original String
		 * @param charsToDelete a set of characters to delete. E.g. "az\n" will delete
		 *                      'a's, 'z's and new lines.
		 * @return the resulting String
		 */
		public static String deleteAny(String inString, String charsToDelete) {
			if (!hasLength(inString) || !hasLength(charsToDelete)) {
				return inString;
			}
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < inString.length(); i++) {
				char c = inString.charAt(i);
				if (charsToDelete.indexOf(c) == -1) {
					sb.append(c);
				}
			}
			return sb.toString();
		}

		// ---------------------------------------------------------------------
		// Convenience methods for working with formatted Strings
		// ---------------------------------------------------------------------

		/**
		 * Quote the given String with single quotes.
		 *
		 * @param str the input String (e.g. "myString")
		 * @return the quoted String (e.g. "'myString'"), or
		 *         <code>null<code> if the input was <code>null</code>
		 */
		public static String quote(String str) {
			return (str != null ? "'" + str + "'" : null);
		}

		/**
		 * Turn the given Object into a String with single quotes if it is a String;
		 * keeping the Object as-is else.
		 *
		 * @param obj the input Object (e.g. "myString")
		 * @return the quoted String (e.g. "'myString'"), or the input object as-is if
		 *         not a String
		 */
		public static Object quoteIfString(Object obj) {
			return (obj instanceof String ? quote((String) obj) : obj);
		}

		/**
		 * Unqualify a string qualified by a '.' dot character. For example,
		 * "this.name.is.qualified", returns "qualified".
		 *
		 * @param qualifiedName the qualified name
		 */
		public static String unqualify(String qualifiedName) {
			return unqualify(qualifiedName, '.');
		}

		/**
		 * Unqualify a string qualified by a separator character. For example,
		 * "this:name:is:qualified" returns "qualified" if using a ':' separator.
		 *
		 * @param qualifiedName the qualified name
		 * @param separator     the separator
		 */
		public static String unqualify(String qualifiedName, char separator) {
			return qualifiedName.substring(qualifiedName.lastIndexOf(separator) + 1);
		}

		/**
		 * Capitalize a <code>String</code>, changing the first letter to upper case as
		 * per {@link Character#toUpperCase(char)}. No other letters are changed.
		 *
		 * @param str the String to capitalize, may be <code>null</code>
		 * @return the capitalized String, <code>null</code> if null
		 */
		public static String capitalize(String str) {
			return changeFirstCharacterCase(str, true);
		}

		/**
		 * Uncapitalize a <code>String</code>, changing the first letter to lower case
		 * as per {@link Character#toLowerCase(char)}. No other letters are changed.
		 *
		 * @param str the String to uncapitalize, may be <code>null</code>
		 * @return the uncapitalized String, <code>null</code> if null
		 */
		public static String uncapitalize(String str) {
			return changeFirstCharacterCase(str, false);
		}

		private static String changeFirstCharacterCase(String str, boolean capitalize) {
			if (str == null || str.length() == 0) {
				return str;
			}
			StringBuilder sb = new StringBuilder(str.length());
			if (capitalize) {
				sb.append(Character.toUpperCase(str.charAt(0)));
			} else {
				sb.append(Character.toLowerCase(str.charAt(0)));
			}
			sb.append(str.substring(1));
			return sb.toString();
		}

		/**
		 * Extract the filename from the given path, e.g. "mypath/myfile.txt" ->
		 * "myfile.txt".
		 *
		 * @param path the file path (may be <code>null</code>)
		 * @return the extracted filename, or <code>null</code> if none
		 */
		public static String getFilename(String path) {
			if (path == null) {
				return null;
			}
			int separatorIndex = path.lastIndexOf(FOLDER_SEPARATOR);
			return (separatorIndex != -1 ? path.substring(separatorIndex + 1) : path);
		}

		/**
		 * Extract the filename extension from the given path, e.g. "mypath/myfile.txt"
		 * -> "txt".
		 *
		 * @param path the file path (may be <code>null</code>)
		 * @return the extracted filename extension, or <code>null</code> if none
		 */
		public static String getFilenameExtension(String path) {
			if (path == null) {
				return null;
			}
			int extIndex = path.lastIndexOf(EXTENSION_SEPARATOR);
			if (extIndex == -1) {
				return null;
			}
			int folderIndex = path.lastIndexOf(FOLDER_SEPARATOR);
			if (folderIndex > extIndex) {
				return null;
			}
			return path.substring(extIndex + 1);
		}

		/**
		 * Strip the filename extension from the given path, e.g. "mypath/myfile.txt" ->
		 * "mypath/myfile".
		 *
		 * @param path the file path (may be <code>null</code>)
		 * @return the path with stripped filename extension, or <code>null</code> if
		 *         none
		 */
		public static String stripFilenameExtension(String path) {
			if (path == null) {
				return null;
			}
			int extIndex = path.lastIndexOf(EXTENSION_SEPARATOR);
			if (extIndex == -1) {
				return path;
			}
			int folderIndex = path.lastIndexOf(FOLDER_SEPARATOR);
			if (folderIndex > extIndex) {
				return path;
			}
			return path.substring(0, extIndex);
		}

		/**
		 * Apply the given relative path to the given path, assuming standard Java
		 * folder separation (i.e. "/" separators).
		 *
		 * @param path         the path to start from (usually a full file path)
		 * @param relativePath the relative path to apply (relative to the full file
		 *                     path above)
		 * @return the full file path that results from applying the relative path
		 */
		public static String applyRelativePath(String path, String relativePath) {
			int separatorIndex = path.lastIndexOf(FOLDER_SEPARATOR);
			if (separatorIndex != -1) {
				String newPath = path.substring(0, separatorIndex);
				if (!relativePath.startsWith(FOLDER_SEPARATOR)) {
					newPath += FOLDER_SEPARATOR;
				}
				return newPath + relativePath;
			} else {
				return relativePath;
			}
		}

		/**
		 * Parse the given <code>localeString</code> value into a {@link Locale}.
		 * <p>
		 * This is the inverse operation of {@link Locale#toString Locale's toString}.
		 *
		 * @param localeString the locale string, following <code>Locale's</code>
		 *                     <code>toString()</code> format ("en", "en_UK", etc); also
		 *                     accepts spaces as separators, as an alternative to
		 *                     underscores
		 * @return a corresponding <code>Locale</code> instance
		 */
		public static Locale parseLocaleString(String localeString) {
			String[] parts = tokenizeToStringArray(localeString, "_ ", false, false);
			if (parts == null) {
				return null;
			}
			String language = (parts.length > 0 ? parts[0] : "");
			String country = (parts.length > 1 ? parts[1] : "");
			validateLocalePart(language);
			validateLocalePart(country);
			String variant = "";
			if (parts.length >= 2) {
				// There is definitely a variant, and it is everything after the country
				// code sans the separator between the country code and the variant.
				int endIndexOfCountryCode = localeString.lastIndexOf(country) + country.length();
				// Strip off any leading '_' and whitespace, what's left is the variant.
				variant = trimLeadingWhitespace(localeString.substring(endIndexOfCountryCode));
				if (variant != null && !variant.equals("") && variant.startsWith("_")) {
					variant = trimLeadingCharacter(variant, '_');
				}
			}
			if (variant == null){
				variant = "";
			}
			return (language.length() > 0 ? new Locale(language, country, variant) : null);
		}

		private static void validateLocalePart(String localePart) {
			for (int i = 0; i < localePart.length(); i++) {
				char ch = localePart.charAt(i);
				if (ch != '_' && ch != ' ' && !Character.isLetterOrDigit(ch)) {
					throw new IllegalArgumentException("Locale part \"" + localePart + "\" contains invalid characters");
				}
			}
		}

		/**
		 * Determine the RFC 3066 compliant language tag, as used for the HTTP
		 * "Accept-Language" header.
		 *
		 * @param locale the Locale to transform to a language tag
		 * @return the RFC 3066 compliant language tag as String
		 */
		public static String toLanguageTag(Locale locale) {
			return locale.getLanguage() + (hasText(locale.getCountry()) ? "-" + locale.getCountry() : "");
		}

		// ---------------------------------------------------------------------
		// Convenience methods for working with String arrays
		// ---------------------------------------------------------------------

		/**
		 * Copy the given Collection into a String array. The Collection must contain
		 * String elements only.
		 *
		 * @param collection the Collection to copy
		 * @return the String array (<code>null</code> if the passed-in Collection was
		 *         <code>null</code>)
		 */
		public static String[] toStringArray(Collection<String> collection) {
			if (collection == null) {
				return null;
			}
			return collection.toArray(new String[collection.size()]);
		}

		/**
		 * Copy the given Enumeration into a String array. The Enumeration must contain
		 * String elements only.
		 *
		 * @param enumeration the Enumeration to copy
		 * @return the String array (<code>null</code> if the passed-in Enumeration was
		 *         <code>null</code>)
		 */
		public static String[] toStringArray(Enumeration<String> enumeration) {
			if (enumeration == null) {
				return null;
			}
			List<String> list = Collections.list(enumeration);
			return list.toArray(new String[list.size()]);
		}

		/**
		 * Split a String at the first occurrence of the delimiter. Does not include the
		 * delimiter in the result.
		 *
		 * @param toSplit   the string to split
		 * @param delimiter to split the string up with
		 * @return a two element array with index 0 being before the delimiter, and
		 *         index 1 being after the delimiter (neither element includes the
		 *         delimiter); or <code>null</code> if the delimiter wasn't found in the
		 *         given input String
		 */
		public static String[] split(String toSplit, String delimiter) {
			if (!hasLength(toSplit) || !hasLength(delimiter)) {
				return null;
			}
			int offset = toSplit.indexOf(delimiter);
			if (offset < 0) {
				return null;
			}
			String beforeDelimiter = toSplit.substring(0, offset);
			String afterDelimiter = toSplit.substring(offset + delimiter.length());
			return new String[] { beforeDelimiter, afterDelimiter };
		}

		/**
		 * Tokenize the given String into a String array via a StringTokenizer. Trims
		 * tokens and omits empty tokens.
		 * <p>
		 * The given delimiters string is supposed to consist of any number of delimiter
		 * characters. Each of those characters can be used to separate tokens. A
		 * delimiter is always a single character; for multi-character delimiters,
		 * consider using <code>delimitedListToStringArray</code>
		 *
		 * @param str        the String to tokenize
		 * @param delimiters the delimiter characters, assembled as String (each of
		 *                   those characters is individually considered as delimiter).
		 * @return an array of the tokens
		 * @see java.util.StringTokenizer
		 * @see java.lang.String#trim()
		 * @see #delimitedListToStringArray
		 */
		public static String[] tokenizeToStringArray(String str, String delimiters) {
			return tokenizeToStringArray(str, delimiters, true, true);
		}

		/**
		 * Tokenize the given String into a String array via a StringTokenizer.
		 * <p>
		 * The given delimiters string is supposed to consist of any number of delimiter
		 * characters. Each of those characters can be used to separate tokens. A
		 * delimiter is always a single character; for multi-character delimiters,
		 * consider using <code>delimitedListToStringArray</code>
		 *
		 * @param str               the String to tokenize
		 * @param delimiters        the delimiter characters, assembled as String (each
		 *                          of those characters is individually considered as
		 *                          delimiter)
		 * @param trimTokens        trim the tokens via String's <code>trim</code>
		 * @param ignoreEmptyTokens omit empty tokens from the result array (only
		 *                          applies to tokens that are empty after trimming;
		 *                          StringTokenizer will not consider subsequent
		 *                          delimiters as token in the first place).
		 * @return an array of the tokens (<code>null</code> if the input String was
		 *         <code>null</code>)
		 * @see java.util.StringTokenizer
		 * @see java.lang.String#trim()
		 * @see #delimitedListToStringArray
		 */
		public static String[] tokenizeToStringArray(String str, String delimiters, boolean trimTokens,
													 boolean ignoreEmptyTokens) {

			if (str == null) {
				return null;
			}
			StringTokenizer st = new StringTokenizer(str, delimiters);
			List<String> tokens = new ArrayList<String>();
			while (st.hasMoreTokens()) {
				String token = st.nextToken();
				if (trimTokens) {
					token = token.trim();
				}
				if (!ignoreEmptyTokens || token.length() > 0) {
					tokens.add(token);
				}
			}
			return toStringArray(tokens);
		}

		/**
		 * Take a String which is a delimited list and convert it to a String array.
		 * <p>
		 * A single delimiter can consists of more than one character: It will still be
		 * considered as single delimiter string, rather than as bunch of potential
		 * delimiter characters - in contrast to <code>tokenizeToStringArray</code>.
		 *
		 * @param str       the input String
		 * @param delimiter the delimiter between elements (this is a single delimiter,
		 *                  rather than a bunch individual delimiter characters)
		 * @return an array of the tokens in the list
		 * @see #tokenizeToStringArray
		 */
		public static String[] delimitedListToStringArray(String str, String delimiter) {
			return delimitedListToStringArray(str, delimiter, null);
		}

		/**
		 * Take a String which is a delimited list and convert it to a String array.
		 * <p>
		 * A single delimiter can consists of more than one character: It will still be
		 * considered as single delimiter string, rather than as bunch of potential
		 * delimiter characters - in contrast to <code>tokenizeToStringArray</code>.
		 *
		 * @param str           the input String
		 * @param delimiter     the delimiter between elements (this is a single
		 *                      delimiter, rather than a bunch individual delimiter
		 *                      characters)
		 * @param charsToDelete a set of characters to delete. Useful for deleting
		 *                      unwanted line breaks: e.g. "\r\n\f" will delete all new
		 *                      lines and line feeds in a String.
		 * @return an array of the tokens in the list
		 * @see #tokenizeToStringArray
		 */
		public static String[] delimitedListToStringArray(String str, String delimiter, String charsToDelete) {
			if (str == null) {
				return new String[0];
			}
			if (delimiter == null) {
				return new String[] { str };
			}
			List<String> result = new ArrayList<String>();
			if ("".equals(delimiter)) {
				for (int i = 0; i < str.length(); i++) {
					result.add(deleteAny(str.substring(i, i + 1), charsToDelete));
				}
			} else {
				int pos = 0;
				int delPos;
				while ((delPos = str.indexOf(delimiter, pos)) != -1) {
					result.add(deleteAny(str.substring(pos, delPos), charsToDelete));
					pos = delPos + delimiter.length();
				}
				if (str.length() > 0 && pos <= str.length()) {
					// Add rest of String, but not in case of empty input.
					result.add(deleteAny(str.substring(pos), charsToDelete));
				}
			}
			return toStringArray(result);
		}

		/**
		 * Convert a CSV list into an array of Strings.
		 *
		 * @param str the input String
		 * @return an array of Strings, or the empty array in case of empty input
		 */
		public static String[] commaDelimitedListToStringArray(String str) {
			return delimitedListToStringArray(str, ",");
		}

		/**
		 * Convenience method to convert a CSV string list to a set. Note that this will
		 * suppress duplicates.
		 *
		 * @param str the input String
		 * @return a Set of String entries in the list
		 */
		public static Set<String> commaDelimitedListToSet(String str) {
			Set<String> set = new TreeSet<String>();
			String[] tokens = commaDelimitedListToStringArray(str);
			for (String token : tokens) {
				set.add(token);
			}
			return set;
		}

	}
}